==> jupyter-0.11.0-43-snapshot.jar: ksql/gen/protocol.clj <==
(ns ksql.gen.protocol
  (:require [clojure.pprint]))


(def ^:dynamic gcontext {})

(defn new-step
  ([subject ref]
   (new-step subject "" ref))
  ([subject property ref]
   [subject property ref]))


(defn add-step [acc new-step-v]
  (conj acc new-step-v)
  )



(defprotocol MappingStore
  (-get-all-mapping [this] "return all mapping")
  (-push-mapping [this mapping-coll] "push new mapping ")
  (-peek-mapping [this] "peek mapping")
  (-pull-mapping [this] "pull mapping"))






(defn peek-mapping [store]
  (-peek-mapping store)
  )


(defn pull-mapping [store]
  (-pull-mapping store)
  )

(defn push-mapping [store v]
  (-push-mapping store v)
  )

(defn get-all-mapping [store]
  (-get-all-mapping store)
  )




(def arvo-p-type #{"string" "bytes" "double" "float" "long" "int" "boolean" "null"})


(defmulti read-req (fn [t v] t))

(defmulti gen-ksql :ksql-gen-type)

(def gen-ksql-rekey-stream-from-mapping "rekey")
(def gen-ksql-stream-insert-from-mapping "insert-into")
(def gen-ksql-stream-from-mapping "stream-to-stream")
(def gen-ksql-create-stream "create-stream")
(def gen-ksql-table-from-stream-after-rekey "table-from-stream-after-rekey")
(def gen-ksql-create-table-as "create-table-as")
(def gen-ksql-create-table "create-table")

(defmulti invoke-dataflow-impl (fn [op-name context request] op-name))

(defmulti expand-step (fn [_ _ mapping-fields]
                        ;(println "--" mapping-fields)
                        (get-in mapping-fields [2 0] #_(get mapping-fields :transfer_fn))))

(defmethod expand-step :default
  [_ _ w]
  ;(vector m)
  [w]
  )


(defrecord ExpandTerminate [v])

(defn new-expand-terminate [v]
  (ExpandTerminate. v))

(defn get-expand-terminate-value [t]
  (get t :v))

(defn is-terminate-step [v]
  (instance? ExpandTerminate v)
  )


(comment

  (-> (new-expand-terminate "hello")
      (get-expand-terminate-value)
      )


  (-> {}                                                    ;(new-expand-terminate "hello")
      (is-terminate-step)
      ;      (get-expand-terminate-value)
      )

  )

(defmulti expand-flow (fn [_ mapping-fields] (first (get (first mapping-fields) :transfer_fn))))



(defmethod expand-flow :default
  [_ mapping]
  ;(println "--" (count mapping) "--" mapping)
  [(new-expand-terminate mapping)]
  ;(new-expand-terminate [mapping] )
  )



(defmulti transfer-fn (fn [f]
                        (-> (first f)
                            (clojure.string/trim)
                            (clojure.string/lower-case))))


(defmulti gen-connector (fn [m]
                          ;(get m t)
                          (get-in m [:template :config :connector.class])

                          ))

(defmethod gen-connector
  :default
  [connector-schema]
  ; (  connector-schema)
  (get connector-schema :template))



(defn log-v
  ([v] (log-v "" v))
  ([msg v]
   (println "--start--" msg)
   (clojure.pprint/pprint v)
   (println "--end--")
   v)

  )


(defprotocol GetValue
  (-getValue [_]))

(defrecord ConstantValue [v]
  GetValue
  (-getValue [_] v))


(defn new-constant [v]
  (->ConstantValue v))


(defn constant-string? [v]
  (instance? ConstantValue v))


(defrecord RefValue [v]
  GetValue
  (-getValue [_] v))


(defrecord QuoteStringValue [v]
  GetValue
  (-getValue [_] v))

(defn new-quote-string [v]
  (->QuoteStringValue v))


(defn quote-string? [v]
  (instance? QuoteStringValue v))

(defn get-quote-value [v]
  (get v :v)
  )




(defn as-referance [v]
  (->RefValue v))



(defn get-value [v-type]
  (-getValue v-type))


(comment

  ;(get-value (as-constant 23))

  )


(def msg {:100 "Could not find this reference on current context "
          :500 "Key is missing in mapping or source stream/table "

          })


(defn get-source-schema [md-repo source-name]
  (reduce (fn [acc schema]
            (if (= (get schema :sink-name) source-name)
              (reduced schema)
              acc
              )
            ) nil md-repo))


(defn get-all-subject-names [md-repo]
  (into [] (map :sink-name) md-repo))


(defn get-object-value [mapping]
  (into [] (rest (get (first mapping) :transfer_fn [[]] ))))


(defn get-property-names [md-repo source-name]
  (if-let [w (get-source-schema md-repo source-name)]
    (into [] (map :name ) (get w :fields))
    []))

==> jupyter-0.11.0-46-snapshot.jar: ksql/gen/protocol.clj <==
(ns ksql.gen.protocol
  (:require [clojure.pprint]))


(def ^:dynamic gcontext {})

(defn new-step
  ([subject ref]
   (new-step subject "" ref))
  ([subject property ref]
   [subject property ref]))


(defn add-step [acc new-step-v]
  (conj acc new-step-v)
  )



(defprotocol MDRepo
  (-get-all-metadata [this] "return all mapping")
  (-push-metadata [this mapping-coll] "push new mapping ")
  (-peek-metadata [this] "peek mapping")
  (-pull-metadata [this] "pull mapping"))






(defn peek-metadata [store]
  (-peek-metadata store)
  )


(defn pull-metadata [store]
  (-pull-metadata store)
  )

(defn push-metadata [store v]
  (-push-metadata store v)
  )

(defn get-all-metadata [store]
  (-get-all-metadata store)
  )




(def arvo-p-type #{"string" "bytes" "double" "float" "long" "int" "boolean" "null"})


(defmulti read-req (fn [t v] t))

(defmulti gen-ksql :ksql-gen-type)

(def gen-ksql-rekey-stream-from-mapping "rekey")
(def gen-ksql-stream-insert-from-mapping "insert-into")
(def gen-ksql-stream-from-mapping "stream-to-stream")
(def gen-ksql-create-stream "create-stream")
(def gen-ksql-table-from-stream-after-rekey "table-from-stream-after-rekey")
(def gen-ksql-create-table-as "create-table-as")
(def gen-ksql-create-table "create-table")

(defmulti invoke-dataflow-impl (fn [op-name context request] op-name))

(defmulti expand-step (fn [_ _ mapping-fields]
                        ;(println "--" mapping-fields)
                        (get-in mapping-fields [2 0] #_(get mapping-fields :transfer_fn))))

(defmethod expand-step :default
  [_ _ w]
  ;(vector m)
  [w]
  )


(defrecord ExpandTerminate [v])

(defn new-expand-terminate [v]
  (ExpandTerminate. v))

(defn get-expand-terminate-value [t]
  (get t :v))

(defn is-terminate-step [v]
  (instance? ExpandTerminate v)
  )


(comment

  (-> (new-expand-terminate "hello")
      (get-expand-terminate-value)
      )


  (-> {}                                                    ;(new-expand-terminate "hello")
      (is-terminate-step)
      ;      (get-expand-terminate-value)
      )

  )

(defmulti expand-flow (fn [_ mapping-fields] (first (get (first mapping-fields) :transfer_fn))))



(defmethod expand-flow :default
  [_ mapping]
  ;(println "--" (count mapping) "--" mapping)
  [(new-expand-terminate mapping)]
  ;(new-expand-terminate [mapping] )
  )



(defmulti transfer-fn (fn [f]
                        (-> (first f)
                            (clojure.string/trim)
                            (clojure.string/lower-case))))


(defmulti gen-connector (fn [m]
                          ;(get m t)
                          (get-in m [:template :config :connector.class])

                          ))

(defmethod gen-connector
  :default
  [connector-schema]
  ; (  connector-schema)
  (get connector-schema :template))



(defn log-v
  ([v] (log-v "" v))
  ([msg v]
   (println "--start--" msg)
   (clojure.pprint/pprint v)
   (println "--end--")
   v)

  )


(defprotocol GetValue
  (-getValue [_]))

(defrecord ConstantValue [v]
  GetValue
  (-getValue [_] v))


(defn new-constant [v]
  (->ConstantValue v))


(defn constant-string? [v]
  (instance? ConstantValue v))


(defrecord RefValue [v]
  GetValue
  (-getValue [_] v))


(defrecord QuoteStringValue [v]
  GetValue
  (-getValue [_] v))

(defn new-quote-string [v]
  (->QuoteStringValue v))


(defn quote-string? [v]
  (instance? QuoteStringValue v))

(defn get-quote-value [v]
  (get v :v)
  )




(defn as-referance [v]
  (->RefValue v))



(defn get-value [v-type]
  (-getValue v-type))


(comment

  ;(get-value (as-constant 23))

  )


(def msg {:100 "Could not find this reference on current context "
          :500 "Key is missing in mapping or source stream/table "

          })


(defn get-source-schema [md-repo source-name]
  (reduce (fn [acc schema]
            (if (= (get schema :sink-name) source-name)
              (reduced schema)
              acc
              )
            ) nil md-repo))


(defn get-all-subject-names [md-repo]
  (into [] (map :sink-name) md-repo))


(defn get-object-value [mapping]
  (into [] (rest (get (first mapping) :transfer_fn [[]] ))))


(defn get-property-names [md-repo source-name]
  (if-let [w (get-source-schema md-repo source-name)]
    (into [] (map :name ) (get w :fields))
    []))
